#!/usr/bin/perl

use warnings;
use strict;

use Socket qw(:DEFAULT AI_CANONNAME IPPROTO_TCP IPPROTO_UDP IPPROTO_RAW SOCK_STREAM SOCK_DGRAM SOCK_RAW getaddrinfo inet_ntop inet_pton);

# map protocol number to name
sub pprotocol {
  my ($proto) = @_;
  if ($proto == IPPROTO_TCP) {
    return 'IPPROTO_TCP';
  } elsif ($proto == IPPROTO_UDP) {
    return 'IPPROTO_UDP';
  } else {
    return 'n/a';
  }
}

# map socket type number to name
sub psocktype {
  my ($socktype) = @_;
  if ($socktype == SOCK_STREAM) {
    return 'SOCK_STREAM';
  } elsif ($socktype == SOCK_DGRAM) {
    return 'SOCK_DGRAM';
  } elsif ($socktype == SOCK_RAW) {
    return 'SOCK_RAW';
  } else {
    return 'unknown';
  }
}

die "Must specify name to resolve" if (not $ARGV[0] and not $ARGV[1]);

my $name = $ARGV[0] or undef;
my $service = $ARGV[1] or undef;

# we want the canonical name on the first entry returned
my $hints = {};

# filter by socket type (here TCP)
$hints->{socktype} = SOCK_STREAM;
if ($ARGV[0]) {
  $hints->{flags} = AI_CANONNAME;
}

my ($err, @addrs) = getaddrinfo ($name, $service, $hints);

die "getaddrinfo: error or no results" if $err;

# If we get here, each element of @addrs is a hash
# reference with the following keys (addrinfo struct members):

# 'family'      (AF_INET, AF_INET6)
# 'protocol'    (IPPROTO_TCP, IPPROTO_UDP)
# 'canonname'   (Only if requested with the AI_CANONNAME flag, and only on the first entry)
# 'addr'        This is a sockaddr (_in or _in6 depending on the address family above)
# 'socktype'    (SOCK_STREAM, SOCK_DGRAM, SOCK_RAW)

# dump results
for(@addrs) {

  my ($canonname, $protocol, $socktype) = (($_->{canonname} or ""), pprotocol($_->{protocol}), psocktype($_->{socktype}));

  if ($_->{family} == AF_INET) {

    # port is always 0 when resolving a hostname
    my ($port, $addr4) = sockaddr_in($_->{addr});

    print "IPv4:\n";
    print "  " . inet_ntop(AF_INET, $addr4) . ", port: $port, protocol: $_->{protocol} ($protocol), socktype: $_->{socktype} ($socktype), canonname: $canonname\n";
  } else {

    my ($port, $addr6, $scope_id, $flowinfo) = sockaddr_in6($_->{addr});
    print "IPv6:\n";
    print "  " . inet_ntop(AF_INET6, $addr6) . ", port: $port, protocol: $_->{protocol} ($protocol), socktype: $_->{socktype} ($socktype), (scope id: $scope_id, flowinfo: $flowinfo), canonname: $canonname\n";
  }
}

